#!/usr/bin/env python
__all__ = ("main",)


def main():
    """
    Tool to deploy a Heroku Review App in Bitbucket Pipelines.


    Environment Variables used from Bitbucket Pipelines:
        - BITBUCKET_COMMIT: commit hash to be used as the deploy version
        - BITBUCKET_BRANCH: git branch to be used as project name

    Environment Variables needed:
        - Path to the project
        - settings for the Heroku Platform API (HEROKU_API_KEY)
        - Bitbucket Pipelines variables (HEROKU_PIPELINE_NAME)
        - Heroku Project Name (HEROKU_DEV_NAME)

    Make sure you include this envs in the pipeline:
        - HEROKU_API_KEY: Heroku api key
        - HEROKU_PROJECT_NAME: Heroku

    https://devcenter.heroku.com/articles/heroku-postgres-backups#direct-database-to-database-copies
    """
    import os
    from pathlib import Path

    from dotenv import load_dotenv

    from .create_review_app import create_review_app
    from .pipeline import get_pipeline_id
    from .source_blob import create_source_blob
    from .utils import filter_env_vars, get_config_vars

    base_dir = Path.cwd()
    env_path = base_dir / ".env"
    load_dotenv(dotenv_path=env_path)

    base_url = "https://api.heroku.com"

    api_key = os.getenv("HEROKU_API_KEY")
    branch = os.getenv("BITBUCKET_BRANCH")
    project = os.getenv("HEROKU_DEV_NAME")
    version = os.getenv("BITBUCKET_COMMIT")
    heroku_pipeline_name = os.getenv("HEROKU_PIPELINE_NAME")

    headers = {
        "Accept": "application/vnd.heroku+json; version=3",
        "Authorization": f"Bearer {api_key}",
    }

    pipeline = get_pipeline_id(base_url, headers, heroku_pipeline_name)
    source_blob_url = create_source_blob(base_url, headers, base_dir, project, version)
    envs = {**filter_env_vars("CI_"), **get_config_vars(base_url, headers, project)}

    data = {
        "branch": branch,
        "pipeline": pipeline,
        "source_blob": {
            "url": source_blob_url,
            "version": version,
        },
        "environment": envs,
    }

    create_review_app(base_url, headers, data)


if __name__ == "__main__":  # pragma: no cover
    main()



import os

import requests


def filter_env_vars(prefix):
    """
    Matches all environment variables with a certain prefix.
    Returns a dict with the key of the variable without the prefix.

    Example:
        env vars:
            - CI_API_KEY=key
            - CI_URL=https://httpbin.org
            - BASE_URL=https://example.com
        prefix:
            - CI_

        result:
            - {'API_KEY': "key", 'URL': "https://httpbin.org"}
    """
    return {
        k.lstrip(prefix): os.getenv(k)
        for k in dict(os.environ).keys()
        if k.startswith(prefix)
    }


def get_config_vars(base_url, headers, app_name):
    """ Returns a dict with the config vars from a Heroku app """
    url = f"{base_url}/apps/{app_name}/config-vars"

    res = requests.get(url, headers=headers)
    res.raise_for_status()

    return {
        k: v for k, v in res.json().items() if k not in ["DATABASE_URL", "REDIS_URL"]
    }



import requests


def create_review_app(base_url, headers, data):
    url = f"{base_url}/review-apps"

    headers["Content-Type"] = "application/json"

    print("Creating Review app...")
    res = requests.post(url, headers=headers, json=data)
    res.raise_for_status()

    print("Done!")
    response = res.json()

    return response



import requests


def get_pipeline_id(base_url, headers, pipeline_name):
    """Get a Heroku pipeline id based in on a name"""

    url = f"{base_url}/pipelines/{pipeline_name}"

    print("Getting the Pipeline name...")
    res = requests.get(url, headers=headers)
    res.raise_for_status()

    print("Done!")
    print()
    return res.json().get("id")



import subprocess

import requests


def create_archive(path, tag):
    """ create the artifact to upload to Heroku Source API """

    name = f"source_blob_{tag}.tar.gz"

    print("Zipping the project...")
    subprocess.run(["git", "archive", "-o", name, "HEAD"])

    print("Done!")
    print()
    return name


def create_source_blob(base_url, headers, base_dir, project_name, version):
    """
    Upload the a gzipped tar archive of source code
    Necessary for creating a new Review App

    Returns a url with the archive for build.
    """
    url = f"{base_url}/apps/{project_name}/sources"

    res = requests.post(url, headers=headers)
    res.raise_for_status()

    source_blob = res.json().get("source_blob", {})
    get_url = source_blob.get("get_url")
    put_url = source_blob.get("put_url")

    source_blob_filename = create_archive(base_dir, version)

    print("Uploading zipped project...")
    res = requests.put(
        put_url, headers={"Content-Type": ""}, data=open(source_blob_filename, "rb")
    )
    res.raise_for_status()

    print("Done!")
    print()
    return get_url
